<?php

namespace Drupal\block_visibility_groups\Plugin\Condition;

use Drupal\block_visibility_groups\GroupEvaluator;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Entity\DependencyTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Executable\ExecutableManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a 'Condition Group' condition.
 *
 * @Condition(
 *   id = "condition_group",
 *   label = @Translation("Condition group"),
 * )
 */
class ConditionGroup extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  use DependencyTrait;

  /**
   * Groups condition AND operator.
   */
  public const OPERATOR_AND = 'AND';

  /**
   * Groups condition OR operator.
   */
  public const OPERATOR_OR = 'OR';

  /**
   * The condition plugin manager.
   *
   * @var \Drupal\Core\Executable\ExecutableManagerInterface
   */
  protected $manager;

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * The request stack object.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The condition group evaluator.
   *
   * @var \Drupal\block_visibility_groups\GroupEvaluator
   */
  protected $groupEvaluator;

  /**
   * {@inheritDoc}
   */
  public function __construct(EntityStorageInterface $entity_storage, ExecutableManagerInterface $manager, RequestStack $request_stack, GroupEvaluator $group_evaluator, array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->manager = $manager;
    $this->entityStorage = $entity_storage;
    $this->requestStack = $request_stack;
    $this->groupEvaluator = $group_evaluator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $container->get('block_visibility_groups.group_evaluator');
    return new static(
      $container->get('entity_type.manager')->getStorage('block_visibility_group'),
      $container->get('plugin.manager.condition'),
      $container->get('request_stack'),
      $container->get('block_visibility_groups.group_evaluator'),
      $configuration,
      $plugin_id,
      $plugin_definition);
  }

  /**
   * Evaluates the condition.
   *
   * @return bool
   *   TRUE if the condition has been met, FALSE otherwise.
   */
  public function evaluate() {
    if (empty($this->configuration['block_visibility_groups'])) {
      return TRUE;
    }

    /** @var \Drupal\block_visibility_groups\Entity\BlockVisibilityGroup[] $block_visibility_groups */
    $block_visibility_groups = $this
      ->entityStorage
      ->loadMultiple($this->configuration['block_visibility_groups']);
    foreach ($block_visibility_groups as $block_visibility_group) {
      $result = $this->groupEvaluator->evaluateGroup($block_visibility_group);

      if ($result && $this->configuration['operator'] === self::OPERATOR_OR) {
        return TRUE;
      }

      if (!$result && $this->configuration['operator'] === self::OPERATOR_AND) {
        return FALSE;
      }
    }

    return $this->configuration['operator'] === self::OPERATOR_AND;
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    // @todo Implement summary() method.
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $block_visibility_groups = $this->entityStorage->loadMultiple();
    $options = [];
    foreach ($block_visibility_groups as $type) {
      $options[$type->id()] = $type->label();
    }

    $form['block_visibility_groups'] = [
      '#title' => $this->t('Block Visibility Groups'),
      '#type' => 'checkboxes',
      '#options' => $options,
    ];

    $default = $this->configuration['block_visibility_groups'] ?? [];
    $form['block_visibility_groups']['#default_value'] = $default;

    $form['operator'] = [
      '#type' => 'radios',
      '#options' => [
        self::OPERATOR_AND => $this->t('All visibility groups must pass'),
        self::OPERATOR_OR => $this->t('Only one visibility group must pass'),
      ],
      '#default_value' => $this->configuration['operator'] ?? self::OPERATOR_OR,
    ];

    $form['negate']['#access'] = FALSE;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $user_values = $form_state->getValues();
    foreach ($user_values as $key => $value) {
      if ($key !== 'negate') {
        $this->configuration[$key] = $value;
      }
    }

    // Remove empty selections, to keep the configuration clean.
    $this->configuration['block_visibility_groups'] = array_filter($this->configuration['block_visibility_groups']);

    // Clean up the configuration - if no groups were selected then remove all
    // configuration.
    if (empty($this->configuration['block_visibility_groups'])) {
      $this->configuration = [];
    }

    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $this->dependencies = parent::calculateDependencies();

    if (!empty($this->configuration['block_visibility_groups'])) {
      $groups = $this->entityStorage->loadMultiple($this->configuration['block_visibility_groups']);
      foreach ($groups as $group) {
        $this->addDependency('config', $group->getConfigDependencyName());
      }
    }

    return $this->dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = parent::getCacheTags();

    if (!empty($this->configuration['block_visibility_groups'])) {
      $groups = $this
        ->entityStorage
        ->loadMultiple($this->configuration['block_visibility_groups']);
      foreach ($groups as $group) {
        $tags = Cache::mergeTags($tags, $group->getCacheTags());
      }
    }

    return $tags;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $contexts = parent::getCacheContexts();
    if (!empty($this->configuration['block_visibility_groups'])) {
      $groups = $this->entityStorage
        ->loadMultiple($this->configuration['block_visibility_groups']);
      foreach ($groups as $group) {
        if (!empty($group)) {
          $contexts = Cache::mergeContexts($contexts, $group->getCacheContexts());
        }
      }
    }
    return $contexts;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    $age = parent::getCacheMaxAge();
    if (!empty($this->configuration['block_visibility_groups'])) {
      $groups = $this->entityStorage
        ->loadMultiple($this->configuration['block_visibility_groups']);
      foreach ($groups as $group) {
        if ($group) {
          $age = Cache::mergeMaxAges($age, $group->getCacheMaxAge());
        }
      }
    }
    return $age;
  }

}
